@@ -6,10 +6,11 @@ Build HTML forms
 
 import logging
 from cgi import escape
+import re
 
 from bn import HTMLFragment
 from formbuild.internal import html_open, html_close, \
-   check_attributes, handle_input, group, _checkable, _select
+   check_attributes, handle_input, group, _checkable, _select, _split
 
 log = logging.getLogger(__name__)
 
@@ -34,10 +35,16 @@ class Form(object):
             self.value = {}
         else:
             self.value = value
+            #for k in self.value:
+            #    if not isinstance(self.value[k], unicode):
+            #        raise Exception('Form values must always be a unicode string, the key %r is %r'%(k, self.value[k]))
         if error is None:
             self.error = {}
         else:
             self.error = error
+            #for k in self.error:
+            #    if not isinstance(self.error[k], unicode):
+            #        raise Exception('Error messages must always be a unicode string, the key %r is %r'%(k, self.error[k]))
         if option is None:
             self.option = {}
         else:
@@ -547,7 +554,7 @@ class Form(object):
         attributes = check_attributes(attributes, ['name'])
         attributes["name"] = name
         return html_open('textarea', False, attributes)+\
-           escape(self.value.get(name, u''))+u'</textarea>'
+           escape(self.value.get(name) or u'')+u'</textarea>'
     
     #
     # Single value fields with read-only values set at desgin time
@@ -645,10 +652,10 @@ class Form(object):
     # Single value fields with options
     #
 
-    def dropdown(self, name, attributes=None, get_option_attributes=None):
+    def dropdown(self, name, option=None, attributes=None, get_option_attributes=None):
         """\
         Create a single-valued <select> field
-
+        
         >>> form = Form(
         ...     value={'fruit': u'1'}, 
         ...     option={
@@ -665,10 +672,10 @@ class Form(object):
         <option value="2&gt;">Apples &lt;&gt;</option>
         <option value="3">Pears</option>
         </select>
-
+        
         If not options for the select field are specified in the form
         constructor, no options will be rendered:
-
+        
         >>> form = Form(
         ...     value={'fruit': u'1'}, 
         ...     option={}
@@ -676,8 +683,40 @@ class Form(object):
         >>> print form.dropdown('fruit')
         <select name="fruit">
         </select>
+           
+        Create a single-valued <select> field from nested data with shared options
+           
+        >>> form = Form(
+        ...     value={'fruit[0].id': u'1', 'fruit[1].id': u'3'}, 
+        ...     option={
+        ...         'fruit[*].id': [
+        ...             ('1', 'Bananas'), 
+        ...             ('2>', 'Apples <>'),
+        ...             ('3', 'Pears'),
+        ...         ]
+        ...     }
+        ... )
+        >>> print form.dropdown('fruit[0].id')
+        <select name="fruit[0].id">
+        <option selected="selected" value="1">Bananas</option>
+        <option value="2&gt;">Apples &lt;&gt;</option>
+        <option value="3">Pears</option>
+        </select>
+        >>> print form.dropdown('fruit[1].id')
+        <select name="fruit[1].id">
+        <option value="1">Bananas</option>
+        <option value="2&gt;">Apples &lt;&gt;</option>
+        <option selected="selected" value="3">Pears</option>
+        </select>
         """
         value = self.value.get(name, u'')
+        if '.' in name or '[' in name:
+            parts = name.split('.')
+            name_ = '.'.join(parts[:-1])
+            sub_name = parts[-1]
+            real_option = _get_option(self, option, name_, sub_name=sub_name)
+        else:
+            real_option = self.option.get(name, [])
         if not isinstance(value, (str, unicode)):
             raise Exception(
                 'The value for a dropdown should be a unicode '
@@ -685,9 +724,9 @@ class Form(object):
                     type(value),
                 )
             )
-        return _select(self.value.get(name, []), self.option.get(name, []), False, name, attributes, get_option_attributes, self)
+        return _select(self.value.get(name, []), real_option, False, name, attributes, get_option_attributes, self)
      
-    def radio_group(self, name, align='horiz', cols=4, sub_name=None):
+    def radio_group(self, name, option=None, align='horiz', cols=4, sub_name=None):
         """Radio Group Field.
     
         ``value``
@@ -746,18 +785,40 @@ class Form(object):
         u''
         >>> form.radio_group('fruit', align='table')
         u''
+         
+        Here's an example with nested variables:
+        
+        >>> form = Form(
+        ...     value={'fruit[0].id': '1'}, 
+        ...     option={
+        ...         'fruit[*].id': [
+        ...             ('1', 'Bananas'), 
+        ...             ('2>', 'Apples <>'),
+        ...             ('3', 'Pears'),
+        ...         ]
+        ...     }
+        ... )
+        >>> print form.radio_group('fruit[0].id')
+        <input type="radio" name="fruit[0].id" value="1" checked="checked" /> Bananas
+        <input type="radio" name="fruit[1].id" value="2&gt;" /> Apples &lt;&gt;
+        <input type="radio" name="fruit[2].id" value="3" /> Pears
         """
-        if sub_name:
-            selected_values = []
-            for k, v in self.value.items():
-                if k.startswith(name+'.') or k.startswith(name+'['):
-                    selected_values.append(v)
+        if '.' in name or '[' in name:
+            parts = name.split('.')
+            name_ = '['.join('.'.join(parts[:-1]).split('[')[:-1])
+            sub_name = parts[-1]
+            real_option = _get_option(self, option, name_, sub_name=sub_name)
         else:
-            selected_values = self.value.get(name)
+            name_ = name
+            real_option = self.option.get(name, [])
+        if self.value.get(name, []):
+            selected_values = [self.value[name]]
+        else:
+            selected_values = []
         return group( 
-            name, 
-            self.value.get(name, []), 
-            self.option.get(name, []),
+            name_, 
+            selected_values,
+            real_option,
             'radio',
             align,
             cols,
@@ -801,7 +862,7 @@ class Form(object):
         """
         return _select(self.value.get(name, []), self.option.get(name, []), True, name, attributes, get_option_attributes, self)
         
-    def checkbox_group(self, name, align='horiz', cols=4, sub_name=None):
+    def checkbox_group(self, name, option=None, align='horiz', cols=4, sub_name=None):
         """Check Box Group Field.
     
         ``align``
@@ -856,21 +917,89 @@ class Form(object):
         u''
         >>> form.checkbox_group('fruit', align='table')
         u''
+
+        You can also have sub_name options
+
+        >>> form = Form(
+        ...     value={
+        ...         'fruit[0].id': '1',
+        ...         'fruit[1].id': '3',
+        ...     }, 
+        ...     option={
+        ...         'fruit[0].id': [
+        ...             ('1', 'Bananas'), 
+        ...             ('2>', 'Apples <>'),
+        ...             ('3', 'Pears'),
+        ...         ]
+        ...     }
+        ... )
+        >>> print form.checkbox_group('fruit', sub_name='id')
+        <input type="checkbox" name="fruit[0].id" value="1" checked="checked" /> Bananas
+        <input type="checkbox" name="fruit[1].id" value="2&gt;" /> Apples &lt;&gt;
+        <input type="checkbox" name="fruit[2].id" value="3" checked="checked" /> Pears
+        >>> form = Form(
+        ...     value={
+        ...         'person[0].fruit[0].id': '1',
+        ...         'person[0].fruit[1].id': '3',
+        ...     }, 
+        ...     option={
+        ...         'person[*].fruit[*].id': [
+        ...             ('1', 'Bananas'), 
+        ...             ('2>', 'Apples <>'),
+        ...             ('3', 'Pears'),
+        ...         ]
+        ...     }
+        ... )
+        >>> print form.checkbox_group('person[0].fruit', sub_name='id')
+        <input type="checkbox" name="person[0].fruit[0].id" value="1" checked="checked" /> Bananas
+        <input type="checkbox" name="person[0].fruit[1].id" value="2&gt;" /> Apples &lt;&gt;
+        <input type="checkbox" name="person[0].fruit[2].id" value="3" checked="checked" /> Pears
         """
+        if name.endswith(']') and sub_name:
+            raise Exception('The name should not end with %r when using sub_name'%name[name.rfind('['):])
+        # Format the selected values into the correct flattened structure
         if sub_name:
             selected_values = []
             for k, v in self.value.items():
-                if k.startswith(name+'.') or k.startswith(name+'['):
+                if k.startswith(name+'[') and k.endswith('].'+sub_name):
                     selected_values.append(v)
         else:
             selected_values = self.value.get(name)
+
         return group( 
             name, 
             selected_values, 
-            self.option.get(name, []),
+            _get_option(self, option, name, sub_name),
             'checkbox',
             align,
             cols,
             sub_name
         )
 
+def _get_option(form, option, name, sub_name=None):
+    if option is None:
+        if not sub_name:
+            real_options = form.option.get(name, [])
+        else:
+            # First see if there is an exact match for this key
+            real_options = None
+            for option in form.option:
+                if option == name+'.'+sub_name:
+                    real_options = form.option[option]
+            # Otherwise treat all the keys as regexes and merge the options
+            # of any matching keys
+            found = []
+            if real_options is None:
+                for option in form.option:
+                    key = name+'.'+sub_name
+                    match = re.match(option.replace('[', '\[').replace(']', '\]'), key)
+                    if match is None:
+                        if found:
+                            raise Exception('The option keys %r and %r both match this checkbox group %r'%(found[0], option, key))
+                        else:
+                            found.append(option)
+            if found:
+                real_options = form.option[found[0]]
+    else:
+        real_options = option
+    return real_options
